{% extends "base.html" %}
{% block header %}{% end %}
{% block body %}
 <!--
   Sign-in https://getbootstrap.com/docs/5.3/examples/sign-in/
   MIT License Copyright (c) 2011-2023 The Bootstrap Authors
 -->
 <main class="form-signin w-100 m-auto">
  <form action="{{ handler.settings["login_url"] }}" method="POST">
    <div class="ms-2">
     <img class="logoimg" src="{{ static_url("fletchck.svg") }}" alt="{{ handler.settings["site_name"] }}">
     {% if error %}<span class="badge thought text-bg-warning">{{ error }}</span>{% end %}
    </div>
    <div class="form-floating">
      <input type="text" class="form-control" name="username" id="username" placeholder="username">
      <label for="username">Username</label>
    </div>
    <div class="form-floating">
      <input type="password" class="form-control" name="password" id="password" placeholder="password">
      <label for="password">Password</label>
    </div>
    <button class="btn btn-primary w-100 py-2" type="submit">Login</button>
  </form>
 </main>
{% end %}
