<!doctype html>
<html lang="en">
 <!--
   Fletchck - Machine Monitor
   MIT License Copyright (c) 2023-2025
   Version: {{ handler.settings["site_version"] }}
 -->
 <head>
  <meta charset="utf-8">
  <meta name="viewport"
        content="width=device-width, initial-scale=1">
  <meta name="application-name"
        content="{{ handler.settings["site_name"] }}">
  <meta name="description" content="Machine Monitor">
  <title>{{ handler.settings["site_name"] }}</title>
  <link href="{{ static_url("fletchck.svg") }}"
        rel="icon">
  <link href="{{ static_url("bootstrap.min.css") }}"
        integrity="sha384-QWTKZyjpPEjISv5WaRU9OFeRpok6YctnYmDr5pNlyT2bRjXh0JMhjY6hW+ALEwIH"
        rel="stylesheet">
  <link href="{{ static_url("fletchck.css") }}"
        integrity="sha384-PMc2kmMOzaZsoFgrCe40MaQopctx9THeZ8+etZq1AiM4RQ/dh0OOYvilf7wU2vLB"
        rel="stylesheet">
  {% block header %}{% end %}
 </head>
 <body>
  {% block body %}{% end %}
 </body>
</html>
