{% extends "base.html" %}
{% block header %}
 <script src="{{ static_url("fletchck.js") }}"
         integrity="sha384-AAeY5dyjo6i2R+1luRxWRFt3ntnDANR2UJDencfML8n/VWSaG7XP4UNjJ1HCz/mp"
         defer="defer"></script>
{% end %}
{% block body %}
<header class="container">
 <div class="ms-2">
 <a href="/" title="{{ handler.settings["site_name"] }}"><img class="logoimg" src="{{ static_url("fletchck.svg") }}" alt="{{ handler.settings["site_name"] }}"></a>
     {% if status['info'] %}<span class="badge thought text-bg-warning">{{ status['info'] }}</span>{% end %}
 </div>
 <ul class="nav nav-underline nav-justified">
  <li class="nav-item"><a href="/checks" class="nav-link{{ ' active' if section == 'check' else ''}}">Checks</a></li>
  <li class="nav-item"><a href="/config" class="nav-link{{ ' active' if section == 'config' else ''}}">Config</a></li>
  <li class="nav-item"><a href="/log" class="nav-link{{ ' active' if section == 'log' else ''}}">Log</a></li>
  <li class="nav-item"><a href="/logout" class="nav-link">Sign Out</a></li>
 </ul>
</header>

<main class="container">
{% block main %}{% end %}
</main>
{% end %}
